/*
 *Wiggle Circle Animator v0.1
 *Peter Barnum <pbarnum@cs.cmu.edu>
 *
 *Copyright (c) 2010, Peter Barnum
 *
 *Distributed under the MIT license:
 *
 *Permission is hereby granted, free of charge, to any person obtaining a copy
 *of this software and associated documentation files (the "Software"), to deal
 *in the Software without restriction, including without limitation the rights
 *to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *copies of the Software, and to permit persons to whom the Software is
 *furnished to do so, subject to the following conditions:
 *
 *The above copyright notice and this permission notice shall be included in
 *all copies or substantial portions of the Software.
 *
 *THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *THE SOFTWARE.
 */

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;

/**
 * This is the core unit of any Wiggle Circle animation. The Circle class holds
 * the position, diameter, and color of a given circle, and renders it when
 * passed the appropriate FrequencyParameters for the CircleGroup it belongs to.
 * 
 * @author Peter Barnum
 * @version 0.1
 * @since 0.1
 */
public class Circle {
    public Point position;
    public int diameter;
    Color color;

    public Circle(Point position, int diameter, Color color){
        this.position = position;
        this.diameter = diameter;
        this.color = color;
    }

    public void paint(Graphics g, double phase, FrequencyParameters horizontalFreqParams, FrequencyParameters verticalFreqParams, int outlineType, boolean paused){
        Graphics2D g2d = (Graphics2D) g;

        //System.err.println("(paused1 " + " " + outlineType + " " + paused + ") ");
        double hAmplitude = horizontalFreqParams.amplitude;
        if(!horizontalFreqParams.amplitudeChecked){ hAmplitude = 0.0; }
        double hPhaseOffset = horizontalFreqParams.phaseOffset;
        if(!horizontalFreqParams.phaseOffsetChecked){ hPhaseOffset = 0.0; }
        double hSpatialFreq = horizontalFreqParams.spatialFreq;
        if(!horizontalFreqParams.spatialFreqChecked){ hSpatialFreq = 10e300; }
        double hTemporalFreq = horizontalFreqParams.temporalFreq;
        if(!horizontalFreqParams.temporalFreqChecked){ hTemporalFreq = 0.0; }
        double vAmplitude = verticalFreqParams.amplitude;
        if(!verticalFreqParams.amplitudeChecked){ vAmplitude = 0.0; }
        double vPhaseOffset = verticalFreqParams.phaseOffset;
        if(!verticalFreqParams.phaseOffsetChecked){ vPhaseOffset = 0.0; }
        double vSpatialFreq = verticalFreqParams.spatialFreq;
        if(!verticalFreqParams.spatialFreqChecked){ vSpatialFreq = 10e300; }
        double vTemporalFreq = verticalFreqParams.temporalFreq;
        if(!verticalFreqParams.temporalFreqChecked){ vTemporalFreq = 0.0; }

        double positionX = (double) position.x;
        double positionY = (double) position.y;

        double currentHPhase = 0;
        if(!paused){
            //System.err.println("(paused2 " + " " + outlineType + " " + paused + ") ");
            if(!horizontalFreqParams.spatialFreqChecked){
                currentHPhase = phase + .5 * hPhaseOffset * Math.PI/180.0;
            } else{
                currentHPhase = phase + positionX / hSpatialFreq /(.5*Math.PI) + .5*hPhaseOffset * Math.PI/180.0;
            }
        }

        double currentVPhase = 0;
        if(!paused){
            if(!verticalFreqParams.spatialFreqChecked){
                currentVPhase = phase + .5 * vPhaseOffset * Math.PI/180.0;
            } else{
                currentVPhase = phase + positionY / vSpatialFreq /(.5*Math.PI) + .5*vPhaseOffset * Math.PI/180.0;
            }
        }

        int trueX = (int) (positionX-diameter/2 +
                           vAmplitude * Math.sin(vTemporalFreq * currentVPhase));
        int trueY = (int)(positionY-diameter/2 +
                          hAmplitude * Math.sin(hTemporalFreq * currentHPhase));

        if(outlineType == 0){
            g2d.setStroke(new BasicStroke(1F));
            g2d.setColor(color);
            g2d.fillOval(trueX, trueY, diameter, diameter);
        }else if(outlineType == 1){
            g2d.setStroke(new BasicStroke(3F));
            g2d.setColor(Color.black);
            g2d.drawOval(trueX, trueY, diameter+2, diameter+2);
        }else if(outlineType == 2){
            g2d.setStroke(new BasicStroke(3F));
            g2d.setColor(Color.white);
            g2d.drawOval(trueX, trueY, diameter+1, diameter+1);
        }
    }
}
