/*
 *Wiggle Circle Animator v0.1
 *Peter Barnum <pbarnum@cs.cmu.edu>
 *
 *Copyright (c) 2010, Peter Barnum
 *
 *Distributed under the MIT license:
 *
 *Permission is hereby granted, free of charge, to any person obtaining a copy
 *of this software and associated documentation files (the "Software"), to deal
 *in the Software without restriction, including without limitation the rights
 *to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *copies of the Software, and to permit persons to whom the Software is
 *furnished to do so, subject to the following conditions:
 *
 *The above copyright notice and this permission notice shall be included in
 *all copies or substantial portions of the Software.
 *
 *THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *THE SOFTWARE.
 */

import java.awt.Color;
import java.awt.Point;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * LoadXML parses a Wiggle Circle animation in XML format.
 * Based on the example at http://www.mkyong.com/java/how-to-read-xml-file-in-java-sax-parser/
 *
 * @author Peter Barnum
 * @version 0.1
 * @since 0.1
 */
public class LoadXML {
    public void parseFile(String filename, CircleGroupManager circleGroupManager){
        try{
            SAXParserFactory factory = SAXParserFactory.newInstance();
            SAXParser saxParser = factory.newSAXParser();
            MyDefaultHandler myDefaultHandler = new MyDefaultHandler(circleGroupManager, true);

            circleGroupManager.clear();
            saxParser.parse(filename, myDefaultHandler);
        } catch (Exception e) {
            e.printStackTrace();
            circleGroupManager.clear();
        }
    }

    public void parseString(String inputString, CircleGroupManager circleGroupManager){
        try{
            SAXParserFactory factory = SAXParserFactory.newInstance();
            SAXParser saxParser = factory.newSAXParser();
            MyDefaultHandler myDefaultHandler = new MyDefaultHandler(circleGroupManager, true);

            circleGroupManager.clear();

            InputStream is = new ByteArrayInputStream(inputString.getBytes("UTF-8"));

            saxParser.parse(is, myDefaultHandler);

            circleGroupManager.curCircleGroupInd = 0;
        } catch (Exception e) {
            e.printStackTrace();
            circleGroupManager.clear();
        }
    }
}

/**
 * MyDefaultHandler is a helper function for LoadXML, to load and parse an
 * XML string.
 *
 * @author Peter Barnum
 * @version 0.1
 * @since 0.1
 */
class MyDefaultHandler extends DefaultHandler {
    CircleGroupManager circleGroupManager;
    CircleGroup curCircleGroup;
    boolean displayDebug;

    public MyDefaultHandler(CircleGroupManager circleGroupManager, boolean displayDebug){
        this.circleGroupManager = circleGroupManager;
        this.displayDebug = displayDebug;
    }

    @Override
    public void startElement(String uri, String localName,
        String qName, Attributes attributes)
        throws SAXException {
        
        if(qName.equals("Group")){
            FrequencyParameters horizontalFreqParam = new FrequencyParameters();
            FrequencyParameters verticalFreqParams = new FrequencyParameters();

            horizontalFreqParam.amplitude = Integer.decode(attributes.getValue("hAmplitude"));
            horizontalFreqParam.spatialFreq = Integer.decode(attributes.getValue("hSpatialFreq"));
            horizontalFreqParam.phaseOffset = Integer.decode(attributes.getValue("hPhaseOffset"));
            horizontalFreqParam.temporalFreq = Integer.decode(attributes.getValue("hTemporalFreq"));

            horizontalFreqParam.amplitudeChecked = Boolean.parseBoolean(attributes.getValue("hAmplitudeChecked"));
            horizontalFreqParam.spatialFreqChecked = Boolean.parseBoolean(attributes.getValue("hSpatialFreqChecked"));
            horizontalFreqParam.phaseOffsetChecked = Boolean.parseBoolean(attributes.getValue("hPhaseOffsetChecked"));
            horizontalFreqParam.temporalFreqChecked = Boolean.parseBoolean(attributes.getValue("hTemporalFreqChecked"));

            verticalFreqParams.amplitude = Integer.decode(attributes.getValue("vAmplitude"));
            verticalFreqParams.spatialFreq = Integer.decode(attributes.getValue("vSpatialFreq"));
            verticalFreqParams.phaseOffset = Integer.decode(attributes.getValue("vPhaseOffset"));
            verticalFreqParams.temporalFreq = Integer.decode(attributes.getValue("vTemporalFreq"));

            verticalFreqParams.amplitudeChecked = Boolean.parseBoolean(attributes.getValue("vAmplitudeChecked"));
            verticalFreqParams.spatialFreqChecked = Boolean.parseBoolean(attributes.getValue("vSpatialFreqChecked"));
            verticalFreqParams.phaseOffsetChecked = Boolean.parseBoolean(attributes.getValue("vPhaseOffsetChecked"));
            verticalFreqParams.temporalFreqChecked = Boolean.parseBoolean(attributes.getValue("vTemporalFreqChecked"));

            curCircleGroup = new CircleGroup(horizontalFreqParam, verticalFreqParams);

            if(displayDebug){
                System.err.println("Group (" + curCircleGroup.horizontalFreqParams.amplitude + ")");                
            }
        }else if(qName.equals("Circle")){
            String[] positionText = attributes.getValue("position").split(",");
            String[] colorText = attributes.getValue("color").split(",");

            Point position = new Point(Integer.decode(positionText[0]),Integer.decode(positionText[1]));
            int diameter = Integer.decode(attributes.getValue("diameter"));
            Color color = new Color(Integer.decode(colorText[0]),
                                    Integer.decode(colorText[1]),
                                    Integer.decode(colorText[2]));

            curCircleGroup.addCircle(position, diameter, color);
        }
     }

    @Override
    public void endElement(String uri, String localName,
          String qName)
          throws SAXException {

        if(qName.equals("Group")){
            circleGroupManager.addCircleGroup(curCircleGroup);
            //System.err.println("adding group " +  circleGroupManager.circleGroups.size());
        }
     }
}
